﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [Item("PhenotypicSimilarityCalculator", "An operator that calculates the similarity betweeon two trees based on the correlation of their outputs.")]
  [StorableClass]
  public class PhenotypicSimilarityCalculator : SingleObjectiveSolutionSimilarityCalculator {
    public PhenotypicSimilarityCalculator(PhenotypicSimilarityCalculator original, Cloner cloner)
      : base(original, cloner) {
    }

    public PhenotypicSimilarityCalculator() { }

    [StorableConstructor]
    protected PhenotypicSimilarityCalculator(bool deserializing) : base(deserializing) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PhenotypicSimilarityCalculator(this, cloner);
    }

    public override double CalculateSolutionSimilarity(IScope leftSolution, IScope rightSolution) {
      var leftValues = (DoubleArray)leftSolution.Variables["EstimatedValues"].Value;
      var rightValues = (DoubleArray)rightSolution.Variables["EstimatedValues"].Value;
      OnlineCalculatorError error;
      var r2 = OnlinePearsonsRSquaredCalculator.Calculate(leftValues, rightValues, out error);

      if (r2 > 1.0)
        r2 = 1.0; // sometimes due to fp errors it can happen that the r2 is over 1 (like 1.0000000009)

      return error != OnlineCalculatorError.None ? 0 : r2;
    }
  }
}
